package com.soss.system.service.impl;

import com.soss.common.enums.ShopState;
import com.soss.common.utils.StringUtils;
import com.soss.system.domain.Shop;
import com.soss.system.mapper.ShopMapper;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

@Service
public class AppServiceImpl {
    @Autowired
    private ShopMapper shopMapper;
    private static final double EARTH_RADIUS = 6378137;

    public Shop getShop(String lng, String lat, Boolean testFlag) {
        Shop shop = new Shop();
        if (BooleanUtils.isTrue(testFlag)) {
            shop.setStates(Arrays.asList(ShopState.TESTING.getState(), ShopState.OPEN.getState()));
        } else {
            shop.setState(ShopState.OPEN.getState());
        }
        shop.setIsDefault(1L);
        List<Shop> shops = shopMapper.selectShopList(shop);
        if (CollectionUtils.isEmpty(shops)) {
            return new Shop();
        }

        shops.forEach(sp -> {
            double realDistance;
            if (StringUtils.isNotEmpty(lng) && StringUtils.isNotEmpty(lat)) {
                realDistance = getRealDistance(Double.parseDouble(lng), Double.parseDouble(lat), Double.parseDouble(sp.getLng()), Double.parseDouble(sp.getLat()));
            } else {
                realDistance = Double.MAX_VALUE;
            }
            sp.setRealDistance(realDistance);
            sp.setDistance(getDistance(realDistance));
        });
        shops.sort(Comparator.comparing(Shop::getRealDistance).reversed());
        return shops.get(0);
    }

    public static String getDistance(Double realDistance) {
        if (realDistance == null) {
            return "-1";
        }
        if (realDistance.equals(Double.MAX_VALUE)) {
            return "-1";
        }

        if (realDistance > 1000 && realDistance < 10000) {
            DecimalFormat df = new DecimalFormat("######0.0");
            return df.format(realDistance / 1000) + "km";
        } else if (realDistance >= 10000) {
            Double aDouble = realDistance / 1000;
            return aDouble.intValue() + "km";
        }

        return realDistance.intValue() + "m";
    }

    /**
     * 根据两点间经纬度坐标（double值），计算两点间距离，单位为米
     */
    public static String getDistance(double lng1, double lat1, double lng2, double lat2) {
        double s = getRealDistance(lng1, lat1, lng2, lat2);
        return getDistance(s);
    }

    public static double getRealDistance(double lng1, double lat1, double lng2, double lat2) {
        double radLat1 = Math.toRadians(lat1);
        double radLat2 = Math.toRadians(lat2);
        double a = radLat1 - radLat2;
        double b = Math.toRadians(lng1) - Math.toRadians(lng2);
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) +
                Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s *= EARTH_RADIUS;
        s = Math.round(s * 10000) / 10000;
        return s;
    }
}
